<?

 /*************************************************************************************
 * Filename: backend.php
 *
 * This file is part of the Dashboard JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The Dashboard JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/dashboard/
 *************************************************************************************/


 require_once($_path["include"]."data.php");
 require_once($_path["include"]."cms.php");
 require_once($_path["include"]."toolkit.php");

 class Plugin_Backend_dashboard extends Base_Plugin_Backend
 {
  function main()
  {
   $this->tk=new Toolkit();
   unset($this->tk->e);
   $this->tk->e=&$this->e;

   if(request("snmpd"))
    if(!$this->snmpdConfig(request("snmpd")))
     return false;

   return true;
  }

  function output()
  {
   $output="";

   if(request("cms"))
    $output.=$this->cms();
   else
   {
    if(request("test"))
     $output.=$this->connectionTest(request("test"));

    $output.=$this->overview();
   }

   return $output;
  }

  function snmpdConfig($server)
  {
   $d=Data::cache("dashboard_servers", "view", "item");
   $d->setShowFields("username", "password", "passphrase");
   $d->setValue("nr", $server);

   if(!($snmpdConfig=@file_get_contents($this->plugin->path["root"]."misc/snmpd.conf")))
   {
    $this->e->add("Unable to read: ".$this->plugin->path["root"]."misc/snmpd.conf");
    return false;
   }

   if(!$d->get())
   {
    $this->e->add($d->e);
    return false;
   }
   else if(!$d->next())
   {
    $this->e->add("Server not found");
    return false;
   }

   if($d->v["password"])
    $secLevel="Auth";
   else
    $secLevel="NoAuth";

   if($d->v["passphrase"])
    $secLevel.="Priv";
   else
    $secLevel.="NoPriv";

   $snmpdConfig=str_replace("\$USERNAME", $d->v["username"], $snmpdConfig);
   $snmpdConfig=str_replace("\$SECURITYLEVEL", $secLevel, $snmpdConfig);

   header('Content-type: text/plain');
   header('Content-length: '.strlen($snmpdConfig));
   header("Content-Disposition: attachment; filename=\"snmpd.conf\"");

   print $snmpdConfig;
   exit;
  }

  function connectionTest($server)
  {
   $d=Data::cache("dashboard_servers", "view", "item");
   $d->setShowFields("name", "hostname");
   $d->setValue("nr", $server);

   $r="<h3>Connection test</h3>\n";
   $r.="<div class=\"connectionTest\">\n";

   if(!$d->get())
    $r.=$d->e->show();
   else if(!$d->next())
   {
    $this->e->add("Server not found");
    $r.=$this->e->show();
   }
   else
   {
    $dc=Data::cache("dashboard_uptime", "view", "all", Array("db" => "dashboard_cli://?".$d->v["name"]));

    if(!$dc->get())
     $r.=$dc->e->show();
    else
    {
     $this->e->add("Server connection test to: \"".$d->v["name"]."\" (".$d->v["hostname"].") successful", "note");
     $r.=$this->e->show();
    }

   }

   $r.="<br/></div>\n";
   return $r;
  }

  function overview()
  {
   $d=Data::cache("dashboard_pluginConfigs", "view", "all");
   $d->setValue("plugin", $this->plugin->name);
   $d->sortOrder("key");
   $d->get(1);

   $r="";

   $r.="<h3>Disk temperature</h3>\n<ul class=\"list decimal\">";
   $r.="<li>Smartctl is used to gather the disk temperature. Since this program requires root privileges to run make sure you add the following line to the \"/etc/sudoers\" file on the applicable host(s):<br/>apache ALL=NOPASSWD:/usr/sbin/smartctl -A -i /dev/sda</li>\n";
   $r.="<li>You'll need to add a line for each disk you've configured, replacing /dev/sda with the applicable device</li>\n";
   $r.="<li>You'll need to replace \"apache\" with the user running the webserver or the SSH user connecting to the host</li>\n";
   $r.="<li>Note that this is not required for SNMP since the daemon already has root privileges</li>\n";
   $r.="</ul><br/>\n";

   $r.="<h3>SNMP</h3>\n<ul class=\"list decimal\">";
   $r.="<li>When using the SNMP method to gather data the configuration file (/etc/snmp/snmpd.conf) will need to be updated. You can download the server specific configuration file below.</li>\n";
   $r.="<li>Additionally the user will need to be added to the secondary configuration file (/var/lib/net-snmp/snmpd.conf). Copy/past the server specific createUser line below.</li>\n";
   $r.="</ul><br/>\n";

   $r.="<h2>".$d->title.($d->r("add")?" <a href=\"".$this->plugin->url()."cms=1&amp;a=add&amp;m=".$d->module."&amp;plugin[0]=".$this->plugin->name."\" title=\"Add ".$d->title."\">".$this->tk->icon("add", "small")."</a>\n":"")."</h2>\n";
   $r.="<div class=\"configs\">\n";

   while($d->next())
   {
    $r.="<div class=\"config\">";
    $r.="<h3>".$d->v["key"]."</h3>";

    $info=Array();
    if($d->v["system"])
     $info[]=$d->title("system");
    if($d->v["uptime"])
     $info[]=$d->title("uptime");
    if($d->v["memory"])
     $info[]=$d->title("memory");
    if($d->v["network"])
     $info[]=$d->title("network");
    if($d->v["diskspace"])
     $info[]=$d->title("diskspace");
    if($d->v["diskstats"])
     $info[]=$d->title("diskstats");
    if($d->v["sensors"])
     $info[]=$d->title("sensors");

    $r.="<div class=\"field\"><div class=\"title\">Display</div><div class=\"input\">".join("<br/>", $info)."</div></div>\n";

    if($d->v["dashboard_pluginConfigsServers"])
    {
     $servers=Array();
     foreach($d->v["dashboard_pluginConfigsServers"] as $server)
      $servers[]="<a href=\"#".$server["name"]."\" title=\"".$server["name"]."\">".$server["name"]."</a> (".$server["hostname"].")";

     $r.="<div class=\"field\"><div class=\"title\">".$d->title("dashboard_pluginConfigsServers")."</div><div class=\"input\">".join("<br/>", $servers)."</div></div>\n";
    }

    if($d->v["dashboard_pluginConfigsAllowedServers"])
    {
     $servers=Array();
     foreach($d->v["dashboard_pluginConfigsAllowedServers"] as $server)
      $servers[]="<a href=\"#".$server["name"]."\" title=\"".$server["name"]."\">".$server["name"]."</a> (".$server["hostname"].")";

     $r.="<div class=\"field\"><div class=\"title\">".$d->title("dashboard_pluginConfigsAllowedServers")."</div><div class=\"input\">".join("<br/>", $servers)."</div></div>\n";
    }

    $r.="<div class=\"cmsOptions\">\n";
    if($d->r("edit")) $r.="<a href=\"".$this->plugin->url()."cms=1&amp;a=edit&amp;m=".$d->module."&amp;nr=".$d->v["nr"]."\" title=\"Edit: ".$d->v["key"]."\">".$this->tk->icon("edit", "small")."</a>\n";
    if($d->r("move")) $r.="<a href=\"".$this->plugin->url()."cms=1&amp;a=move&amp;m=".$d->module."&amp;nr=".$d->v["nr"]."\" title=\"Move: ".$d->v["key"]."\">".$this->tk->icon("move", "small")."</a>\n";
    if($d->r("delete")) $r.="<a href=\"".$this->plugin->url()."cms=1&amp;a=delete&amp;m=".$d->module."&amp;nr=".$d->v["nr"]."\" title=\"Delete: ".$d->v["key"]."\">".$this->tk->icon("delete", "small")."</a>\n";
    $r.="</div>\n";

    $r.="<br/><br/></div>\n";
   }


   //servers
   $d=Data::cache("dashboard_servers", "view", "all");
   $d->sortOrder("name");
   $d->get(0);

   $r.="<h2>".$d->title.($d->r("add")?" <a href=\"".$this->plugin->url()."cms=1&amp;a=add&amp;m=".$d->module."&amp;plugin[0]=".$this->plugin->name."\" title=\"Add ".$d->title."\">".$this->tk->icon("add", "small")."</a>\n":"")."</h2>\n";
   $r.="<div class=\"server\">\n";

   while($d->next())
   {
    $r.="<div class=\"server\">";
    $r.="<h3><a name=\"".$d->v["name"]."\">".$d->v["name"]."</a> (".$d->v["hostname"].")</h3>";

    $info=Array();
    if($d->v["system"])
     $info[]=$d->title("system");
    if($d->v["uptime"])
     $info[]=$d->title("uptime");
    if($d->v["memory"])
     $info[]=$d->title("memory");

    $r.="<div class=\"field\"><div class=\"title\">Information</div><div class=\"input\">".join("<br/>", $info)."</div></div>\n";

    if($d->v["networkDevices"])
     $r.="<div class=\"field\"><div class=\"title\">".$d->title("networkDevices")."</div><div class=\"input\">".join(", ", preg_replace("/^\d+ /", "", $d->v["networkDevices"]))."</div></div>\n";
    if($d->v["diskspaceDevices"])
     $r.="<div class=\"field\"><div class=\"title\">".$d->title("diskspaceDevices")."</div><div class=\"input\">".join(", ", preg_replace("/^\d+ /", "", $d->v["diskspaceDevices"]))."</div></div>\n";
    if($d->v["diskstatsDevices"])
     $r.="<div class=\"field\"><div class=\"title\">".$d->title("diskstatsDevices")."</div><div class=\"input\">".join(", ", preg_replace("/^\d+ /", "", $d->v["diskstatsDevices"]))."</div></div>\n";
    if($d->v["sensorsDevices"])
     $r.="<div class=\"field\"><div class=\"title\">".$d->title("sensorsDevices")."</div><div class=\"input\">".join(", ", preg_replace("/^\d+ /", "", $d->v["sensorsDevices"]))."</div></div>\n";

    if($d->v["method"]=="SNMP")
    {
     $r.="<div class=\"field\"><div class=\"title\">SNMP</div><div class=\"input\">".$d->v["username"]."@".$d->v["hostname"].":".$d->v["port"]."</div></div>\n";

     $r.="<div class=\"field\"><div class=\"title\">snmpd.conf</div><div class=\"input\">";
     $r.="Add into or replace \"<div class=\"hilite\">/etc/snmp/snmpd.conf</div>\" on ".$d->v["hostname"]." the content of \"<a href=\"".$this->plugin->url()."snmpd=".$d->v["nr"]."\" title=\"Download ".$d->v["name"]." snmpd.conf\">snmpd.conf</a>\".<br/>";

     if($d->r("edit")) //show real user/pass/pass for users with edit rights
      $r.="Add \"<div class=\"hilite\">createUser ".$d->v["username"].($d->v["password"]?" MD5 ".$d->v["password"]:"").($d->v["passphrase"]?" DES ".$d->v["passphrase"]:"")."</div>\" to \"<div class=\"hilite\">/var/lib/net-snmp/snmpd.conf</div>\" on ".$d->v["hostname"].".<br/>";
     else
     {
      $r.="Add \"<div class=\"hilite\">createUser username".($d->v["password"]?" MD5 password":"").($d->v["passphrase"]?" DES passphrase":"")."</div>\" to \"<div class=\"hilite\">/var/lib/net-snmp/snmpd.conf</div>\" on ".$d->v["hostname"].".<br/>";
      $r.="Replace \"<div class=\"hilite\">username</div>\", \"<div class=\"hilite\">password</div>\" and \"<div class=\"hilite\">passphrase</div>\" where applicable.<br/>";
     }

     $r.="Note that you'll need to restart snmpd before the changes will take effect.";
     $r.="</div></div>\n";

     $r.="<div class=\"field\"><div class=\"title\">".$d->title("password")." / ".$d->title("passphrase")."</div><div class=\"input\">".($d->v["password"]?"Yes":"<div class=\"hilite\">No</div>")." / ".($d->v["passphrase"]?"Yes":"<div class=\"hilite\">No</div>")."</div></div>\n";
    }
    else if($d->v["method"]=="SSH")
     $r.="<div class=\"field\"><div class=\"title\">SSH</div><div class=\"input\">".$d->v["username"]."@".$d->v["hostname"].":".$d->v["port"]."</div></div>\n";

    $r.="<div class=\"cmsOptions\">\n";
    $r.="<a href=\"".$this->plugin->url()."test=".$d->v["nr"]."\" title=\"Test connection: ".$d->v["name"]."\">".$this->tk->icon("execute", "small")."</a>\n";
    if($d->r("edit")) $r.="<a href=\"".$this->plugin->url()."cms=1&amp;a=edit&amp;m=".$d->module."&amp;nr=".$d->v["nr"]."\" title=\"Edit: ".$d->v["name"]."\">".$this->tk->icon("edit", "small")."</a>\n";
    if($d->r("move")) $r.="<a href=\"".$this->plugin->url()."cms=1&amp;a=move&amp;m=".$d->module."&amp;nr=".$d->v["nr"]."\" title=\"Move: ".$d->v["name"]."\">".$this->tk->icon("move", "small")."</a>\n";
    if($d->r("delete")) $r.="<a href=\"".$this->plugin->url()."cms=1&amp;a=delete&amp;m=".$d->module."&amp;nr=".$d->v["nr"]."\" title=\"Delete: ".$d->v["name"]."\">".$this->tk->icon("delete", "small")."</a>\n";
    $r.="</div>\n";

    $r.="<br/><br/></div>\n";
   }
   $r.="</div>\n";

   return $r;
  }

  function cms()
  {
   $cms=new CMS();

   $hs=new History();
   $hs->cms();

   $cms->addVar("cms", 1);
   foreach($this->plugin->vars as $k => $v)
    $cms->addVar($k, $v);

   $cms->setData(new Data());
   $r=$cms->form();

   return $r;
  }

 }

?>
