<?

 /*************************************************************************************
 * Filename: modules/pluginConfigs.php
 *
 * This file is part of the Dashboard JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The Dashboard JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/dashboard/
 *************************************************************************************/


 require_once($_path["include"]."module.php");

 class Plugin_Module_dashboard_pluginConfigs extends Module
 {
  function __construct(&$data)
  {
   global $_path;

   parent::__construct($data);

   $this->data->l->read("module_dashboard_servers");

   $this->servers=Data::cache("dashboard_servers", "view", "all");
   $this->modules=$this->servers->moduleObject->modules; //use modules definition from dashboard servers

   if($this->data->module!="pluginConfigs")
    $this->extend("pluginConfigs");

   $this->data->title="Dashboard plugin configuration";


   //add module fields
   $groups=Array("Display" => Array());
   foreach($this->modules as $module => $info)
   {
    //get all module fields
    $fields=Array();
    $ds=Data::cache("dashboard_".$module, "view", "item", Array("skipCheckTables" => true));

    foreach($ds->check("use", "show") as $field)
     if($field!="r")
      $fields[$field]=$ds->title($field)." (".$field.")";

    //get default fields
    $defaultFields=Array();
    foreach(array_keys($fields) as $field)
     if(!in_array($field, $info["dontShow"]))
      $defaultFields[]=$field;

    //add fields
    $this->data->addField($module, "boolean", "all", "reload", false, false, $info["use"], $info["title"]);
    $this->data->addField($module."Fields", "multilist", "all", "", 1, 65535, $defaultFields, $info["title"]." display fields");
    $this->data->listOptions($module."Fields", $fields);

    if(isset($info["devices"]))
    {
     $this->data->addField($module."Devices", "multilist", "all", "", 1, 65535, false, $info["title"]." ".$info["devices"]["title"]);
     $this->data->field[$module."Devices"]["min"]=false;

     $groups[$info["title"]]=Array($module."Fields", $module."Devices");
    }
    else
     $groups[$info["title"]]=Array($module."Fields");

    $groups["Display"][]=$module;
   }

   //servers fields
   $this->data->addReference("dashboard_pluginConfigsServers", "dashboard_servers", "nr", "*:*", "nr");
   $this->data->addReference("dashboard_pluginConfigsAllowedServers", "dashboard_servers", "nr", "*:*", "nr");

   if(isset($this->data->field["dashboard_pluginConfigsServers"]))
   {
    $this->data->field["dashboard_pluginConfigsServers"]["title"]="Dashboard servers";
    $this->data->field["dashboard_pluginConfigsServers"]["properties"].=";reload";
    $this->data->field["dashboard_pluginConfigsAllowedServers"]["title"]="Allowed servers";
   }

   $groups["Display"][]="dashboard_pluginConfigsServers";
   $groups["Display"][]="dashboard_pluginConfigsAllowedServers";

   //server graph page mappings
   $this->data->addField("pluginConfigsServerPages", "multilist", "all", "", 0, 65535, false, "Servers graph page mappings");

   //add groups
   foreach($groups as $group => $fields)
    $this->data->group($group, $fields);
  }


  function addPreSubmit(){return $this->checkServers();}
  function copyPreSubmit(){return $this->checkServers();}
  function editPreSubmit(){return $this->checkServers();}

  function checkServers()
  {
   if(!$this->data->getValue("dashboard_pluginConfigsServers"))
   {
    $this->data->e->add("At least 1 server must selected", "error");
    return false;
   }

   return true;
  }


  function postInit()
  {
   if($this->data->action!="add"&&$this->data->action!="copy"&&$this->data->action!="edit")
    return true;

   return $this->customizeFields();
  }
  function addPostGet(){return $this->customizeFields();}
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   if($this->data->vmax==0&&($this->data->action!="add"||$this->data->extra=="copy")) //only change fields if object has data or if add new
    return true;

   if($this->data->vmax)
    $max=$this->data->vmax;
   else
    $max=1;

   //force dashboard plugin
   for($c=0;$c<$max;$c++)
    $this->data->setValue("plugin", "dashboard", $c);

   //get devices for all server selected servers
   $this->servers->clearValues();
   $this->servers->sortOrder("name+");

   $showFields=Array("name");
   foreach(array_keys($this->modules) as $module)
    $showFields[]=$module."Devices";

   $this->servers->setShowFields($showFields);
   unset($showFields);

   //m2m not gathered
   if($this->data->getValue("dashboard_pluginConfigsServers")===false&&($this->data->action=="copy"||$this->data->action=="edit"))
   {
    $d=&$this->data->reference["dashboard_pluginConfigsServers"]["data"];
    $d->setAction("view");
    $d->setExtra("all");
    $d->clearValues();

    $d->setValue("dashboard_pluginConfigs", $this->data->getValue("nr"));

    if(!$d->get())
    {
     $this->data->e->add($d->e);
     return false;
    }

    $this->data->setValue("dashboard_pluginConfigsServers", $d->field["dashboard_servers"]["value"]);
   }

   $c=0;
   foreach($this->data->getValue("dashboard_pluginConfigsServers") as $nr)
   {
    $this->servers->setValue("nr", $nr, $c);
    $this->servers->setOperant("nr", "OR", $c);
    $c++;
   }

   if(!$this->servers->get())
   {
    $this->data->e->add($this->servers->e);
    return false;
   }

   //gather display fields for all modules
   foreach($this->modules as $module => $info)
   {
    //display module fields field
    if($this->data->getValue($module))
     $this->data->resetField($module."Fields");
    else
    {
     $this->data->field[$module."Fields"]["properties"].=";hidden";
     $this->data->field[$module."Fields"]["min"]=false;
    }

    //get devices
    //XXX: note that this will also do a connection test!
    if(isset($info["devices"]))
    {
     //display devices field
     if($this->data->getValue($module))
     {
      $this->data->resetField($module."Devices");
      $this->data->field[$module."Devices"]["min"]=false;
     }
     else
     {
      $this->data->field[$module."Devices"]["properties"].=";hidden";
      $this->data->field[$module."Devices"]["min"]=false;
     }

     $fields=Array();
     while($this->servers->next())
     {
      foreach($this->servers->v[$module."Devices"] as $device)
      {
       $key=$this->servers->v["nr"]." ".$device;
       $value=$this->servers->v["name"]." ".$device;

       $fields[$key]=$value;
      }
     }

     $this->servers->reset();
     $this->data->listOptions($module."Devices", $fields);
    }
   }

   //server graph page mappings
   $this->servers->clearValues();
   if(!$this->servers->get())
   {
    $this->data->e->add($this->servers->e);
    return false;
   }

   //clean up previously added fields
   foreach($this->data->check("reference", "categories") as $field)
   {
    if(strstr($field, "spm_"))
    {
     unset($this->data->reference[$field]);
     unset($this->data->field[$field]);
    }
   }

   //get database values
   $serverPages=Array();
   $this->data->field["pluginConfigsServerPages"]["properties"].=";hidden";
   if($values=$this->data->getValue("pluginConfigsServerPages"))
   {
    foreach($values as $value)
    {
     list($server, $page)=explode("=", $value);
     $serverPages[$server]=$page;
    }
   }

   //add server page fields
   while($this->servers->next())
   {
    $this->data->addField("spm_".$this->servers->v["nr"], "nr", "form", "", false, false, false, $this->servers->v["name"]." graph page");
    $this->data->addReference("spm_".$this->servers->v["nr"], "categories", "nr", "*:0");
    $this->data->field["spm_".$this->servers->v["nr"]]["group"]=$this->data->field["dashboard_pluginConfigsServers"]["group"];

    //no submitted value, use database if available
    $this->data->setRequestValue("spm_".$this->servers->v["nr"]);
    if($this->data->field["spm_".$this->servers->v["nr"]]["valuetype"]=="none")
    {
     if(isset($serverPages[$this->servers->v["nr"]]))
      $this->data->setValue("spm_".$this->servers->v["nr"], $serverPages[$this->servers->v["nr"]]);
     else
      $this->data->setValue("spm_".$this->servers->v["nr"], 0);
    }

   }

   return true;
  }


  function addPreSubmitErrorCheck(){return $this->mergeServerPages();}
  function copyPreSubmitErrorCheck(){return $this->mergeServerPages();}
  function editPreSubmitErrorCheck(){return $this->mergeServerPages();}

  function mergeServerPages()
  {
   $values=Array();
   $this->servers->reset();
   while($this->servers->next())
   {
    if($this->data->getValue("spm_".$this->servers->v["nr"]))
     $values[]=$this->servers->v["nr"]."=".$this->data->getValue("spm_".$this->servers->v["nr"]);

    $this->data->listOptions("pluginConfigsServerPages", $values);
    $this->data->setValue("pluginConfigsServerPages", $values);
   }

   return true;
  }

 }
?>
