<?

 /*************************************************************************************
 * Filename: modules/servers.php
 *
 * This file is part of the Dashboard JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The Dashboard JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/dashboard/
 *************************************************************************************/


 require_once($_path["include"]."module.php");

 class Plugin_Module_dashboard_servers extends Module
 {
  function __construct(&$data)
  {
   global $_path;

   parent::__construct($data);

   $this->modules=Array(
    "system" => Array(
     "title" => "System info",
     "use" => true,
     "dontShow" => Array(
      "hardware", "os", "kernelversion", "distrolastupdate",
     ),
    ),

    "uptime" => Array(
     "title" => "Uptime / Load",
     "use" => true,
     "dontShow" => Array(
      "users_d",
      "load5", "load5_d", "load15", "load15_d",
     ),
    ),

    "memory" => Array(
     "title" => "Memory usage",
     "use" => true,
     "dontShow" => Array(
      "memoryavailable_percentage_d",
      "memoryused_d", "memoryused_percentage", "memoryused_percentage_d",
      "memorycached","memorycached_d", "memorycached_percentage", "memorycached_percentage_d",
      "memorybuffers","memorybuffers_d", "memorybuffers_percentage", "memorybuffers_percentage_d",

      "swapavailable_percentage_d",
      "swapused_d", "swapused_percentage", "swapused_percentage_d",
      "swapcached","swapcached_d", "swapcached_percentage", "swapcached_percentage_d",
     ),
    ),

    "network" => Array(
     "title" => "Network usage",
     "use" => true,
     "devices" => Array(
      "title" => "devices",
      "field" => "device",
      "types" => Array(
       "lan" => "Local network",
       "wan" => "Internet",
       "vpn" => "VPN",
       "wireless" => "Wireless",
      ),
     ),
     "dontShow" => Array(
      "rx", "tx"
     ),
    ),

    "diskspace" => Array(
     "title" => "Disk space",
     "use" => true,
     "devices" => Array(
      "title" => "mount points",
      "field" => "mountpoint",
      "display" => "mountpoint (device)",
      "types" => Array(
       "disk" => "Disk",
       "dvd" => "DVD",
       "cdrom" => "CD-ROM",
       "network" => "Network",
       "special" => "Special",
      ),
     ),
     "dontShow" => Array(
      "available_percentage_d",
      "used_d", "used_percentage", "used_percentage_d",
      "missing", "missing_percentage",
     ),
    ),

    "diskstats" => Array(
     "title" => "Disk statistics",
     "use" => true,
     "devices" => Array(
      "title" => "disks",
      "field" => "deviceattribute",
      "display" => "device model: attributenr attribute (value)",
     ),
     "dontShow" => Array(
      "family", "attributenr", "value_min", "value_max", "deviceattribute",
     ),
    ),

    "sensors" => Array(
     "title" => "Sensors",
     "use" => true,
     "devices" => Array(
      "title" => "devices",
      "field" => "devicesensor",
      "display" => "device: sensor (value unit)",
     ),
     "dontShow" => Array(
      "adapter", "value_min", "value_max", "devicesensor",
     ),
    ),
   );


   $this->data->title="Dashboard servers";

   $this->data->defaultFields();


   //server fields
   $this->data->addField("name", "text", "all", "title", 1, 25, "", "Name");
   $this->data->addField("method", "enum", "all", "reload", 0, false, "Local", "Method");
   $this->data->enumOptions("method", "Local", "SNMP", "SSH");

   $this->data->addField("hostname", "hostname", "all", false, 1, 255, (isset($_SERVER["HTTP_HOST"])?$_SERVER["HTTP_HOST"]:""), "Host name");
   $this->data->addField("port", "nr", "all", false, 1, 65535, 0, "Port number");

   $this->data->addField("username", "title", "all", "reload", 1, 255, "", "User name");
   $this->data->addField("password", "text", "all", "reload", 0, 255, "", "Password");
   $this->data->addField("passphrase", "text", "all", "reload", 0, 255, "", "Pass phrase");


   //add module fields
   $groups=Array("Display" => Array());
   foreach($this->modules as $module => $info)
   {
    //get all module fields
    $fields=Array();
    $ds=Data::cache("dashboard_".$module, "view", "item", Array("skipCheckTables" => true));

    foreach($ds->check("use", "show") as $field)
     if($field!="r")
      $fields[$field]=$ds->title($field)." (".$field.")";

    //get default fields
    $defaultFields=Array();
    foreach(array_keys($fields) as $field)
     if(!in_array($field, $info["dontShow"]))
      $defaultFields[]=$field;

    //add fields
    $this->data->addField($module, "boolean", "all", "reload", false, false, $info["use"], $info["title"]);
    $this->data->addField($module."Fields", "multilist", "all", "", 1, 65535, $defaultFields, $info["title"]." display fields");
    $this->data->listOptions($module."Fields", $fields);

    if(isset($info["devices"]))
    {
     $this->data->addField($module."Devices", "multilist", "all", "", 1, 65535, false, $info["title"]." ".$info["devices"]["title"]);
     $groups[$info["title"]]=Array($module."Fields", $module."Devices");

     //add devices type field
     if(isset($info["devices"]["types"]))
      $this->data->addField($module."DevicesType", "multilist", "show;sql", "", 0, 65535, false, $info["title"]." device types");
    }
    else
     $groups[$info["title"]]=Array($module."Fields");

    $groups["Display"][]=$module;
   }

   //add groups
   foreach($groups as $group => $fields)
    $this->data->group($group, $fields);

  }


  function postInit()
  {
   if($this->data->action!="add"&&$this->data->action!="copy"&&$this->data->action!="edit")
    return true;

   return $this->customizeFields();
  }


  function addPostGet(){return $this->customizeFields();}
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}


  function customizeFields()
  {
   if($this->data->vmax==0&&($this->data->action!="add"||$this->data->extra=="copy")) //only change fields if object has data or if add new
    return true;

   if($this->data->vmax)
    $max=$this->data->vmax;
   else
    $max=1;

   //get db string and set SNMP/SSH fields
   $db="dashboard_cli://";
   if($this->data->getValue("method")=="SNMP")
   {
    $this->data->resetField("hostname");
    $this->data->resetField("port");
    $this->data->resetField("username");

    $this->data->resetField("password");

    if($this->data->getValue("password"))
     $this->data->resetField("passphrase");
    else
    {
     $this->data->field["passphrase"]["properties"].=";hidden";
     $this->data->setValue("passphrase", "");
    }

    //set SNMP port
    if(!$this->data->getValue("port")||$this->data->getValue("port")=="22")
     $this->data->setValue("port", 161);

    if(
     ($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit")&&
     ($this->data->getValue("hostname")&&$this->data->getValue("username")&&$this->data->getValue("port"))
    )
     $db="dashboard_cli://".$this->data->getValue("username").($this->data->getValue("password")?":".$this->data->getValue("password"):"")."@".$this->data->getValue("hostname").":".$this->data->getValue("port")."/snmp/".($this->data->getValue("passphrase")?"?passphrase=".$this->data->getValue("passphrase"):"");
   }
   else if($this->data->getValue("method")=="SSH")
   {
    $this->data->resetField("hostname");
    $this->data->resetField("port");
    $this->data->resetField("username");

    $this->data->field["password"]["properties"].=";hidden";
    $this->data->field["passphrase"]["properties"].=";hidden";

    //set SSH port
    if(!$this->data->getValue("port")||$this->data->getValue("port")=="161")
     $this->data->setValue("port", 22);

    if(
     ($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit")&&
     ($this->data->getValue("hostname")&&$this->data->getValue("username")&&$this->data->getValue("port"))
    )
    $db="dashboard_cli://".$this->data->getValue("username")."@".$this->data->getValue("hostname").":".$this->data->getValue("port")."/ssh/";
   }
   else //local
   {
    $this->data->field["hostname"]["properties"].=";hidden";
    $this->data->field["hostname"]["min"]=false;
    $this->data->field["port"]["properties"].=";hidden";
    $this->data->field["port"]["min"]=false;
    $this->data->field["username"]["properties"].=";hidden";
    $this->data->field["username"]["min"]=false;

    $this->data->field["password"]["properties"].=";hidden";
    $this->data->field["passphrase"]["properties"].=";hidden";
   }

   foreach($this->modules as $module => $info)
   {
    //display module fields field
    if($this->data->getValue($module))
     $this->data->resetField($module."Fields");
    else
    {
     $this->data->field[$module."Fields"]["properties"].=";hidden";
     $this->data->field[$module."Fields"]["min"]=false;
    }

    //get devices
    //XXX: note that this will also do a connection test!
    if(isset($info["devices"]))
    {
     //display devices field
     if($this->data->getValue($module))
      $this->data->resetField($module."Devices");
     else
     {
      $this->data->field[$module."Devices"]["properties"].=";hidden";
      $this->data->field[$module."Devices"]["min"]=false;
     }

     $d=Data::cache("dashboard_".$module, "view", "all", Array("db" => $db));
     $d->clearValues();

     //set up show fields
     $showFields=Array($info["devices"]["field"]);
     if(isset($info["devices"]["display"])&&preg_match_all("/([a-z0-9_]{1,50})/", $info["devices"]["display"], $m))
      for($c=0;$c<sizeof($m[0]);$c++)
       $showFields[]=$m[1][$c];

     $d->setShowFields($showFields);

     if(!$d->get())
     {
      if($this->data->getValue($module))
       $this->data->e->add($d->e);

      continue;
     }

     if($this->data->getValue($module))
      $this->data->e->add($d->e);

     $fields=Array();
     while($d->next())
     {
      $field=$info["devices"]["field"];

      if(isset($info["devices"]["display"]))
      {
       $fields[$d->v[$field]]=$info["devices"]["display"];
       foreach($d->v as $k => $v)
        $fields[$d->v[$field]]=str_replace($k, $v, $fields[$d->v[$field]]);
      }
      else
       $fields[$d->v[$field]]=$d->v[$field];
     }

     $this->data->listOptions($module."Devices", $fields);

     //create devices types fields
     if(isset($info["devices"]["types"]))
     {
      //clean up previously added devices types fields
      foreach($this->data->check("type", "list") as $field)
       if(strstr($field, $module."_dt_"))
        unset($this->data->field[$field]);

      if($this->data->getValue($module))
      {
       //get database values
       $deviceTypes=Array();
       if($values=$this->data->getValue($module."DevicesType"))
       {
        foreach($values as $value)
        {
         if($value)
         {
          list($device, $type)=explode("=", $value);
          $deviceTypes[$device]=$type;
         }
        }
       }

       //add device fields
       foreach($this->data->listOptions[$module."Devices"] as $device => $title)
       {
        $deviceField=$module."_dt_".preg_replace("/[^a-z0-9]/i", "_", $device);
        $this->data->addField($deviceField, "list", "form", "", false, false, false, "Device type ".$title);
        $this->data->listOptions($deviceField, $this->modules[$module]["devices"]["types"]);

        $this->data->field[$deviceField]["group"]=$this->data->field[$module."Devices"]["group"];

        //no submitted value, use database if available
        $this->data->setRequestValue($deviceField);
        if($this->data->field[$deviceField]["valuetype"]=="none")
        {
         if(isset($deviceTypes[$device]))
          $this->data->setValue($deviceField, $deviceTypes[$device]);
         else
          $this->data->setValue($deviceField, "");
        }
       }
      }
     }
    }

   }

   return true;
  }

  function addPreSubmitErrorCheck(){return $this->mergeDevicesTypeFields();}
  function copyPreSubmitErrorCheck(){return $this->mergeDevicesTypeFields();}
  function editPreSubmitErrorCheck(){return $this->mergeDevicesTypeFields();}

  function mergeDevicesTypeFields()
  {
   foreach($this->modules as $module => $info)
   {
    if(isset($info["devices"]["types"]))
    {
     $values=Array();
     foreach($this->data->listOptions[$module."Devices"] as $device => $title)
     {
      $deviceField=$module."_dt_".preg_replace("/[^a-z0-9]/i", "_", $device);

      if(($type=$this->data->getValue($deviceField))&&isset($this->modules[$module]["devices"]["types"][$type]))
       $values[]=$device."=".$type;
     }

     $this->data->listOptions($module."DevicesType", $values);
     $this->data->setValue($module."DevicesType", $values);
    }
   }

   return true;
  }

 }
?>
