

 /*************************************************************************************
 * Filename: script/dashboard.js
 *
 * This file is part of the Dashboard JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The Dashboard JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/dashboard/
 *************************************************************************************/


if(typeof(setGetElementFunctions)!="function")
 alert("Script not loaded: getelements.js");
else if(typeof(onload)!="function")
 alert("Script not loaded: functions.js");
else if(typeof(HTTP)!='function')
 alert("Script not loaded: http.js");
else if(typeof(Flash)!='function')
 alert("Script not loaded: flash.js");
else
 onload("db=new Dashboard();");


function Dashboard()
{
 var self=this;

 self.init=init;

 self.getMinMax=getMinMax;
 self.getLegendColors=getLegendColors;

 self.start=start;
 self.stop=stop;

 self.monitor=monitor;
 self.monitorShow=monitorShow;

 self.hiliteMinMax=hiliteMinMax;
 self.hiliteDelta=hiliteDelta;

 self.decimal=decimal;
 self.humanreadable=humanreadable;


 self.init();
 
 function init() 
 {
  setGetElementFunctions();

  self.updateSpeed=50;

  self.previous=Array();//previous values array

  //find dashboard
  var dashboard=document.getElementsByClassName("dashboard");

  if(dashboard.length==0)
  {
//   alert("Dashboard not found");
   return false;
  }

  dashboard=dashboard[0]; //use first


  //run through all tables to get views
  var viewsCounter=0;
  self.views=Array();

  var tables=dashboard.getElementsByTagName("table");
  for(var c=0;c<tables.length;c++)
  {
   var values=Array();
   var servers=Array();

   var elements=tables[c].getAllElements();
   for(var x=0;x<elements.length;x++)
   {
    if(elements[x].id)
    {
     var m=elements[x].id.match(/^([^_]+)_([^_]+)_(.+)$/);
     var id=elements[x].id;

     values[id]=Array();

     values[id]["type"]=m[1];
     values[id]["server"]=m[2];
     values[id]["field"]=m[3];

     if(m=m[3].match(/^([^\.]+)\.([a-z0-9\.]+)$/i))
     {
      values[id]["field"]=m[1];
      values[id]["device"]=m[2];
     }
     else
      values[id]["device"]=false;

     values[id]["min"]=false;
     values[id]["max"]=false;
     values[id]["min_field"]=false;
     values[id]["max_field"]=false;

     //detect delta image
     var imgs=elements[x].getElementsByTagName("img");
     if(imgs.length&&imgs[0].className.match(/delta/)) //delta image found
     {
      values[id]["deltaImage"]=Array();

      values[id]["deltaImage"]["path"]=imgs[0].src.replace(/[^\/]+$/, "");
      values[id]["deltaImage"]["ext"]=imgs[0].src.replace(/^.*\.([^\.]+)$/, "$1");
      values[id]["deltaImage"]["alt"]=imgs[0].alt;
      values[id]["deltaImage"]["className"]=imgs[0].className;
     }
     else
      values[id]["deltaImage"]=false;

     if(elements[x].className.match(/(^| )humanreadable( |$)/)) //humanreadable
     {
      values[id]["type"]="humanreadable";

      //suffix
      if(m=elements[x].innerHTML.replace(/ /g, "").match(/([^0-9]*)$/)) 
       values[id]["suffix"]=m[1].replace(/^[PTGMK]/, ""); //strip size indicators
      else
       values[id]["suffix"]="";

      //get decimal separator and number of decimcals
      if(m=elements[x].innerHTML.match(/[0-9]+([^0-9])([0-9]+)/)) 
      {
       values[id]["decimalseparator"]=m[1];
       values[id]["decimals"]=m[2].length;
      }
      else
      {
       values[id]["decimalseparator"]=".";
       values[id]["decimals"]=0;
      }
     }
     else if(elements[x].className.match(/(^| )nr( |$)/)) //int
     {
      values[id]["type"]="int";

      //suffix
      if(m=elements[x].innerHTML.replace(/ /g, "").match(/([^0-9]*)$/)) 
       values[id]["suffix"]=m[1];
      else
       values[id]["suffix"]="";
     }
     else if(elements[x].className.match(/(^| )float( |$)/)) //float
     {
      values[id]["type"]="float";

      //suffix
      if(m=elements[x].innerHTML.replace(/ /g, "").match(/([^0-9]*)$/)) 
       values[id]["suffix"]=m[1];
      else
       values[id]["suffix"]="";

      //get decimal separator and number of decimcals
      if(m=elements[x].innerHTML.match(/[0-9]+([^0-9])([0-9]+)/)) 
      {
       values[id]["decimalseparator"]=m[1];
       values[id]["decimals"]=m[2].length;
      }
      else
      {
       values[id]["decimalseparator"]=".";
       values[id]["decimals"]=0;
      }
     }
     else if(elements[x].className.match(/(^| )version( |$)/)) //version
     {
      values[id]["type"]="version";

      //get version separator 
      if(m=elements[x].innerHTML.match(/([^0-9])/)) 
       values[id]["decimalseparator"]=m[1];
      else
       values[id]["decimalseparator"]=".";
     }
     else
      values[id]["type"]="string";

     //graph+device icons prefix
     if(m=elements[x].innerHTML.match(/(\<.+\>)/)) 
      values[id]["icons"]=m[1];
     else
      values[id]["icons"]="";


     if(!servers.in_array(values[id]["server"])) //add server
      servers.push(values[id]["server"]);
    }
   }

   for(var x=0;x<servers.length;x++)
   {
    self.views[viewsCounter]=Array();
    self.views[viewsCounter]["server"]=servers[x];
    self.views[viewsCounter]["type"]=tables[c].className.match(/([^\s]+)/i)[1];
    self.views[viewsCounter]["display"]=tables[c].className.match(/(horizontal|vertical)/)[1];

    //get device field
    if(tables[c].previousSibling&&tables[c].previousSibling.nodeName.match(/input/i)&&tables[c].previousSibling.name=="deviceField")
     self.views[viewsCounter]["deviceField"]=tables[c].previousSibling.value;
    else
     self.views[viewsCounter]["deviceField"]=false;

    self.views[viewsCounter]["values"]=values;

    self.views[viewsCounter]["http"]=new HTTP();
    self.views[viewsCounter]["http"].db("dashboard_cli://?"+servers[x]);

    viewsCounter++;
   }
  }

  setTimeout(function(){self.getMinMax();}, 0); //load min max values in backgound

  self.getLegendColors();

  if(document.getElementById("update").checked)
   return self.start();

  return true;
 }


//GET MIN/MAX

 function getMinMax()
 {
  var http=new HTTP();

  //gather types
  var types=Array();

  for(var c=0;c<self.views.length;c++)
   if(!types.in_array(self.views[c]["type"]))
    types.push(self.views[c]["type"]);

  //get min / max per type
  for(var x=0;x<types.length;x++)
  {
   if(!http.module("dashboard_"+types[x])) //get module definition
   {
    alert(http.error);
    return false;
   }

   //loop through all views to find matching type
   for(var c=0;c<self.views.length;c++)
   {
    if(self.views[c]["type"]==types[x]) //view found
    {
     //loop through all fields to find matching module field 
     for(var k in self.views[c]["values"]) 
     {
      if(k=="in_array") //in_array workaround
       continue;

      var field=self.views[c]["values"][k]["field"];

      http.reset();
      while(v=http.next()) 
      {
       if(typeof(v[field])!="undefined") //module field found
       {
        //set min / max value
        if(v[field]["min"]!==false)
         self.views[c]["values"][k]["min"]=v[field]["min"];

        if(v[field]["max"]!==false)
         self.views[c]["values"][k]["max"]=v[field]["max"];

        //module has $field_min / $field_max fields
        if(typeof(v[field+"_min"])!="undefined") 
         self.views[c]["values"][k]["min_field"]=field+"_min";

        if(typeof(v[field+"_max"])!="undefined") 
         self.views[c]["values"][k]["max_field"]=field+"_max";

        break;
       }
      }
     }
    }
   }
  }

  return true;
 }


//GET LEGEND COLORS

 function getLegendColors()
 {
  self.legendColors=Array();

  var dashboard=document.getElementsByClassName("dashboard")[0];
  var tables=dashboard.getElementsByTagName("table");

  for(var c=0;c<tables.length;c++)
  {
   if(tables[c].className.match(/legend/))
   {
    var elements=tables[c].getElementsByTagName("td");

    for(var x=0;x<elements.length;x++)
    {
     var type=elements[x].className;
     var flash=new Flash(elements[x]); //inits elements[].style 

     self.legendColors[type]=Array();
     self.legendColors[type]["font"]=elements[x].style.color;
     self.legendColors[type]["backgound"]=elements[x].style.backgroundColor;
    }

    return true;
   }
  }

  alert("Legend not found");
  return false;
 }

//START / STOP 

 function start()
 {
  if(self.running) //already running
   return true;

  self.running=true;

  //launch monitoring
  for(var c=0;c<self.views.length;c++)
  {
   self.monitor(c, -1); //do first run to get initial values for delta's, -1 will run it once

   switch(self.views[c]["type"])
   {
    case "system":
    {
     self.monitor(c, 5*60000);
     break;
    }
    case "uptime":
    {
     self.monitor(c, 5000);
     break;
    }
    case "memory":
    {
     self.monitor(c, 1000);
     break;
    }
    case "network":
    {
     self.monitor(c, 1000);
     break;
    }
    case "diskspace":
    {
     self.monitor(c, 10000);
     break;
    }
    case "diskstats":
    {
     self.monitor(c, 5000);
     break;
    }
    case "sensors":
    {
     self.monitor(c, 5000);
     break;
    }
    default: break;
   }
  }
 }

 function stop()
 {
  self.running=false;
  return true;
 }


//MONITOR
 
 function monitor(view, timeout)
 {
  if(!self.running)
   return false;

  //get data
  var http=self.views[view]["http"];
  http.clear();

  //only get relevant fields
  var fields=Array();
  for(k in self.views[view]["values"]) //loop through show fields
  {
   if(k!="in_array") //in_array workaround
   {
    var field=self.views[view]["values"][k]["field"].replace(/(_d|_dps)$/, ""); //strip delta (per second) field suffix
    if(!fields.in_array(field)) //not a show field yet
     fields.push(field); 

    //get $field_min / $field_max fields
    if(self.views[view]["values"][k]["min_field"]&&!fields.in_array(self.views[view]["values"][k]["min_field"]))
     fields.push(self.views[view]["values"][k]["min_field"]);
    if(self.views[view]["values"][k]["max_field"]&&!fields.in_array(self.views[view]["values"][k]["max_field"]))
     fields.push(self.views[view]["values"][k]["max_field"]);
   }
  }

  //get device field
  if(self.views[view]["deviceField"])
   fields.push(self.views[view]["deviceField"]);

  http.show(fields);

  if(!http.data("dashboard_"+self.views[view]["type"], "view", "all", function(){self.monitorShow(view, timeout);}))
//  if(!http.data("dashboard_"+self.views[view]["type"], "view", "all"))
  {
   alert(http.error);
   return false;
  }

//  return self.monitorShow(view, timeout);
  return true;
 }

 function monitorShow(view, timeout)
 {
  if(!self.running)
   return false;

  var http=self.views[view]["http"];

  var fieldPrefix=self.views[view]["type"]+"_"+self.views[view]["server"]+"_";

  //hilite server value
  var obj=document.getElementById(fieldPrefix+"_server");
  if(typeof(obj.flash)=="undefined")
  {
   obj.flash=new Flash(obj);
   obj.flash.speed(self.updateSpeed);
  }

  var e=obj.flash.fontColor(self.legendColors["normal"]["font"], 100, 0);
  if(self.legendColors["normal"]["backgound"])
   obj.flash.backgroundColor(self.legendColors["normal"]["backgound"], 100, e);

  var e=obj.flash.fontColor(self.legendColors["outdated"]["font"], timeout*2, 1);
  if(self.legendColors["outdated"]["backgound"])
   obj.flash.backgroundColor(self.legendColors["outdated"]["backgound"], timeout*2, e);

  obj.flash.go();


  //set values
  while(v=http.next())
  {
   for(field in v)
   {
    if(field=="in_array") //in_array workaround
     continue;

    if(field.match(/_(d|dps)$/)) //delta / delta per second fields
     continue;

    var id=fieldPrefix+field;

    if(self.views[view]["deviceField"]) //add device field suffix
     id+="."+v[self.views[view]["deviceField"]].replace(/[^a-z0-9]/gi, ".").replace(/^\.+/, "");

    var value=v[field].replace(/ /g, "").replace(/,/, ".");

    //set fields value
    if(document.getElementById(id))
    {
     if(value==="")
      document.getElementById(id).innerHTML="&nbsp; ";
     else if(self.views[view]["values"][id]["type"]=="int")
      document.getElementById(id).innerHTML=value+self.views[view]["values"][id]["suffix"];
     else if(self.views[view]["values"][id]["type"]=="float")
      document.getElementById(id).innerHTML=self.decimal(value, self.views[view]["values"][id]["decimals"], self.views[view]["values"][id]["decimalseparator"])+self.views[view]["values"][id]["suffix"];
     else if(self.views[view]["values"][id]["type"]=="humanreadable")
      document.getElementById(id).innerHTML=self.humanreadable(value, self.views[view]["values"][id]["decimals"], self.views[view]["values"][id]["decimalseparator"])+self.views[view]["values"][id]["suffix"];
     else
      document.getElementById(id).innerHTML=v[field];

     //add icons
     if(self.views[view]["values"][id]["icons"])
      document.getElementById(id).innerHTML=self.views[view]["values"][id]["icons"]+" "+document.getElementById(id).innerHTML;

     //min/max hiliting
     if(self.views[view]["values"][id]["type"]=="int"||self.views[view]["values"][id]["type"]=="float"||self.views[view]["values"][id]["type"]=="humanreadable")
     {
      var min, max;

      if(self.views[view]["values"][id]["min_field"]&&
        typeof(v[self.views[view]["values"][id]["min_field"]])!="undefined"&&
        v[self.views[view]["values"][id]["min_field"]]!==false&&
        v[self.views[view]["values"][id]["min_field"]]!==""
       ) //dynamic min value
       min=parseFloat(v[self.views[view]["values"][id]["min_field"]].replace(/ /g, "").replace(/,/, "."));
      else if(self.views[view]["values"][id]["min"]!==false) //static min value
       min=self.views[view]["values"][id]["min"];
      else //no min value
       min=false;

      if(self.views[view]["values"][id]["max_field"]&&
        typeof(v[self.views[view]["values"][id]["max_field"]])!="undefined"&&
        v[self.views[view]["values"][id]["max_field"]]!==false&&
        v[self.views[view]["values"][id]["max_field"]]!==""
       ) //dynamic max value
       max=parseFloat(v[self.views[view]["values"][id]["max_field"]].replace(/ /g, "").replace(/,/, "."));
      else if(self.views[view]["values"][id]["max"]!==false) //static max value
       max=self.views[view]["values"][id]["max"];
      else //no max value
       max=false;

      if(min!==false||max!==false)
       self.hiliteMinMax(view, id, value, min, max, timeout);
     }
    }


    if(typeof(self.previous[id])!="undefined") //previous value found
    {
     //set delta
     var iddelta=fieldPrefix+field+"_d";

     if(self.views[view]["deviceField"]) //add device field suffix
      iddelta+="."+v[self.views[view]["deviceField"]].replace(/[^a-z0-9]/gi, ".").replace(/^\.+/, "");

     if(document.getElementById(iddelta))
     {
      var delta=value-self.previous[id]["value"];

      if(self.views[view]["values"][iddelta]["type"]=="int")
       document.getElementById(iddelta).innerHTML=delta+self.views[view]["values"][iddelta]["suffix"];
      else if(self.views[view]["values"][iddelta]["type"]=="float")
       document.getElementById(iddelta).innerHTML=self.decimal(delta, self.views[view]["values"][iddelta]["decimals"], self.views[view]["values"][iddelta]["decimalseparator"])+self.views[view]["values"][iddelta]["suffix"];
      else //if(self.views[view]["values"][iddelta]["type"]=="humanreadable")
       document.getElementById(iddelta).innerHTML=self.humanreadable(delta, self.views[view]["values"][iddelta]["decimals"], self.views[view]["values"][iddelta]["decimalseparator"])+self.views[view]["values"][iddelta]["suffix"];

      //add delta image
      if(self.views[view]["values"][iddelta]["deltaImage"])
      {
       var img;
       if(delta>0)
        img="up";
       else if(delta<0)
        img="down";
       else
        img="still";

       document.getElementById(iddelta).innerHTML+=" <img src=\""+self.views[view]["values"][iddelta]["deltaImage"]["path"]+img+"."+self.views[view]["values"][iddelta]["deltaImage"]["ext"]+"\" alt=\""+self.views[view]["values"][iddelta]["deltaImage"]["alt"]+" "+img+"\" class=\""+self.views[view]["values"][iddelta]["deltaImage"]["className"]+"\" /> ";
      }

      //min/max hiliting
      var hilited=false;
      if(self.views[view]["values"][iddelta]["min"]!==false||self.views[view]["values"][iddelta]["max"]!==false)
       hilited=self.hiliteMinMax(view, iddelta, delta, self.views[view]["values"][iddelta]["min"], self.views[view]["values"][iddelta]["max"], timeout);
      
      //delta hiliting
      if(!hilited)
       hilited=self.hiliteDelta(view, iddelta, delta, timeout);
     }


     //set delta per second
     var iddelta=fieldPrefix+field+"_dps";

     if(self.views[view]["deviceField"]) //add device field suffix
      iddelta+="."+v[self.views[view]["deviceField"]].replace(/[^a-z0-9]/gi, ".").replace(/^\.+/, "");

     if(document.getElementById(iddelta))
     {
      var delta=value-self.previous[id]["value"];
      delta/=(((new Date).getTime()-self.previous[id]["time"])/1000);

      if(self.views[view]["type"]=="network"&&field=="rx")
       delta=0-delta;

      if(self.views[view]["values"][iddelta]["type"]=="int")
       document.getElementById(iddelta).innerHTML=delta+self.views[view]["values"][iddelta]["suffix"];
      else if(self.views[view]["values"][iddelta]["type"]=="float")
       document.getElementById(iddelta).innerHTML=self.decimal(delta, self.views[view]["values"][iddelta]["decimals"], self.views[view]["values"][iddelta]["decimalseparator"])+self.views[view]["values"][iddelta]["suffix"];
      else //if(self.views[view]["values"][iddelta]["type"]=="humanreadable")
       document.getElementById(iddelta).innerHTML=self.humanreadable(delta, self.views[view]["values"][iddelta]["decimals"], self.views[view]["values"][iddelta]["decimalseparator"])+self.views[view]["values"][iddelta]["suffix"];

      //add delta image
      if(self.views[view]["values"][iddelta]["deltaImage"])
      {
       var img;
       if(delta>0)
        img="up";
       else if(delta<0)
        img="down";
       else
        img="still";

       document.getElementById(iddelta).innerHTML+=" <img src=\""+self.views[view]["values"][iddelta]["deltaImage"]["path"]+img+"."+self.views[view]["values"][iddelta]["deltaImage"]["ext"]+"\" alt=\""+self.views[view]["values"][iddelta]["deltaImage"]["alt"]+" "+img+"\" class=\""+self.views[view]["values"][iddelta]["deltaImage"]["className"]+"\" /> ";
      }

      //min/max hiliting 
      var hilited=false;
      if(self.views[view]["values"][iddelta]["min"]!==false||self.views[view]["values"][iddelta]["max"]!==false)
       hilited=self.hiliteMinMax(view, iddelta, delta, self.views[view]["values"][iddelta]["min"], self.views[view]["values"][iddelta]["max"], timeout);
      
      //delta hiliting
      if(!hilited)
       hilited=self.hiliteDelta(view, iddelta, delta, timeout);
     }
    }
    else
     self.previous[id]=Array(); 

    self.previous[id]["value"]=value; 
    self.previous[id]["time"]=(new Date).getTime(); 
   }
  }

  if(timeout>=0)
   setTimeout(function(){self.monitor(view, timeout);}, timeout);

  return true;
 }


//HILITE

 function hiliteMinMax(view, id, value, min, max, timeout)
 {
  var obj=document.getElementById(id);

  //create flash object
  if(typeof(obj.flash)=="undefined") 
  {
   obj.flash=new Flash(obj);
   obj.flash.speed(self.updateSpeed);
  }


  obj.flash.stop();

  if(min!==false&&value<=min)
  {
//XXX: do transition anyway, deltas could have changed it
//   if(obj.previousMinMax=="min")
//    return false; //no color change

   obj.previousMinMax="min";


   if(obj.style.backgroundColor=="transparent"||!obj.style.backgroundColor) //transparent background
   {
    if(self.legendColors["min"]["backgound"]) //min has background
     obj.flash.backgroundColor("EFEFEF", 0); //set light color for smooth transition
   }

   //min transition   
   var e=obj.flash.fontColor(self.legendColors["min"]["font"], 500);
   if(self.legendColors["min"]["backgound"])
    obj.flash.backgroundColor(self.legendColors["min"]["backgound"], 500, e);

   obj.flash.go();
  }
  else if(max!==false&&value>=max)
  {
//XXX: do transition anyway, deltas could have changed it
//   if(obj.previousMinMax=="max")
//    return false; //no color change

   obj.previousMinMax="max";


   if(obj.style.backgroundColor=="transparent"||!obj.style.backgroundColor) //transparent background
   {
    if(self.legendColors["max"]["backgound"]) //max has background
     obj.flash.backgroundColor("EFEFEF", 0); //set light color for smooth transition
   }

   //max transition   
   var e=obj.flash.fontColor(self.legendColors["max"]["font"], 500);
   if(self.legendColors["max"]["backgound"])
    obj.flash.backgroundColor(self.legendColors["max"]["backgound"], 500, e);

   obj.flash.go();
  }
  else //normal 
  {
   if(obj.previousMinMax=="normal")
    return false; //no color change

   obj.previousMinMax="normal";


   //normal transition
   var e=obj.flash.fontColor(self.legendColors["normal"]["font"], 500);
   if(self.legendColors["normal"]["backgound"]) //background color
    obj.flash.backgroundColor(self.legendColors["normal"]["backgound"], 500, e);
   else if(obj.style.backgroundColor!="transparent"&&obj.style.backgroundColor) //transparent background, but not yet transparent
   {
    obj.flash.backgroundColor("EFEFEF", 500, e); //set light color for smooth transparent
    obj.flash.end('self.obj.style.backgroundColor="transparent";'); //set background transparent at end
   }

   obj.flash.go();
  }

  return true; //color change
 }

 function hiliteDelta(view, id, value, timeout)
 {
  var obj=document.getElementById(id);

  //create flash object
  if(typeof(obj.flash)=="undefined") 
  {
   obj.flash=new Flash(obj);
   obj.flash.speed(self.updateSpeed);
  }


  obj.flash.stop();

  if(value>0) //delta up
  {
//XXX: do transition anyway, min/max could have changed it
//   if(obj.previousDelta=="up")
//    return false; //no color change

   obj.previousDelta="up";


   if(obj.style.backgroundColor=="transparent"||!obj.style.backgroundColor) //transparent background
   {
    if(self.legendColors["deltaup"]["backgound"]) //delta up has background
     obj.flash.backgroundColor("EFEFEF", 0); //set light color for smooth transition
   }

   //delta up transition   
   var e=obj.flash.fontColor(self.legendColors["deltaup"]["font"], timeout/4);
   if(self.legendColors["deltaup"]["backgound"])
    obj.flash.backgroundColor(self.legendColors["deltaup"]["backgound"], timeout/4, e);

   obj.flash.go();

  }
  else if(value<0)
  {
//XXX: do transition anyway, min/max could have changed it
//   if(obj.previousDelta=="down")
//    return false; //no color change

   obj.previousDelta="down";


   if(obj.style.backgroundColor=="transparent"||!obj.style.backgroundColor) //transparent background
   {
    if(self.legendColors["deltadown"]["backgound"]) //delta down has background
     obj.flash.backgroundColor("EFEFEF", 0); //set light color for smooth transition
   }

   //delta down transition   
   var e=obj.flash.fontColor(self.legendColors["deltadown"]["font"], timeout/4);
   if(self.legendColors["deltadown"]["backgound"])
    obj.flash.backgroundColor(self.legendColors["deltadown"]["backgound"], timeout/4, e);

   obj.flash.go();
  }
  else //normal 
  {
   if(obj.previousDelta=="normal")
    return false; //no color change

   obj.previousDelta="normal";

   //normal transition
   var e=obj.flash.fontColor(self.legendColors["normal"]["font"], timeout/4);
   if(self.legendColors["normal"]["backgound"]) //background color
    obj.flash.backgroundColor(self.legendColors["normal"]["backgound"], timeout/4, e);
   else if(obj.style.backgroundColor!="transparent"&&obj.style.backgroundColor) //transparent background, but not yet transparent
   {
    obj.flash.backgroundColor("EFEFEF", timeout/4, e); //set light color for smooth transparent
    obj.flash.end('self.obj.style.backgroundColor="transparent";'); //set background transparent at end
   }

   obj.flash.go();
  }

  return true; //color change
 }


//DECIMAL FORMATTING

 function decimal(number, decimals, decimalseparator)
 {
  number=(number/1).toFixed(decimals);
  return number.replace(/\./, decimalseparator);
 }
 
 function humanreadable(number, decimals, decimalseparator)
 {

  if(-1000000000000000>number||number>1000000000000000)
  {
   number=(number/1000000000000000).toFixed(decimals);
   suffix="P";
  }
  else if(-1000000000000>number||number>1000000000000)
  {
   number=(number/1000000000000).toFixed(decimals);
   suffix="T";
  }
  else if(-1000000000>number||number>1000000000)
  {
   number=(number/1000000000).toFixed(decimals);
   suffix="G";
  }
  else if(-1000000>number||number>1000000)
  {
   number=(number/1000000).toFixed(decimals);
   suffix="M";
  }
  else if(-1000>number||number>1000)
  {
   number=(number/1000).toFixed(decimals);
   suffix="K";
  }
  else
  {
   number=(number/1).toFixed(decimals);
   suffix="";
  }

  return number.replace(/\./, decimalseparator)+suffix;
 }

}
