<?

 /*************************************************************************************
 * Filename: storage/cli.php
 *
 * This file is part of the Dashboard JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The Dashboard JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/dashboard/
 *************************************************************************************/


 require_once($_path["include"]."storage/storage.php");

 class Plugin_Storage_dashboard_cli extends Storage
 {
  function __construct(&$d)
  {
   parent::__construct($d, "dashboard_cli");

   $this->l->read("plugin_dashboard");

   $this->cli=new CLI();

   $this->tables=Array(

    "dashboard_system" => Array(
     "get" => "getSystem"
    ),

    "dashboard_uptime" => Array(
     "get" => "getUptime"
    ),

    "dashboard_diskspace" => Array(
     "get" => "getDiskspace"
    ),

    "dashboard_diskstats" => Array(
     "get" => "getDiskstats"
    ),

    "dashboard_memory" => Array(
     "get" => "getMemory"
    ),

    "dashboard_network" => Array(
     "get" => "getNetwork"
    ),

    "dashboard_sensors" => Array(
     "get" => "getSensors"
    ),
   );
   
   if(!($db=$this->getConfig()))
    return;

   if($db["path"]=="/snmp/") //SNMP
   {
    $this->method="snmp";
    $this->hostname=$db["host"];
    $this->port=$db["port"];
    $this->username=$db["user"];
    $this->password=$db["pass"];
    $this->passphrase=(isset($db["vars"]["passphrase"])?$db["vars"]["passphrase"]:false);
   }
   else if($db["path"]=="/ssh/") //SSH
   {
    $this->method="ssh";

    if(!$this->cli->ssh($db["host"], $db["user"], $db["port"]))
    {
     $this->e->add("Setting CLI to SSH failed", "system");
     $this->e->add($this->cli->e);
    }
   }
   else
    $this->method="local";

   if($this->method=="snmp") //SNMP
   {
    $this->execs=Array(
     "snmpget" => "snmpget",
     "snmpset" => "snmpset",
    );
   }
   else //SSH / local
   {
    $this->execs=Array(
     "echo" => "echo",
     "cat" => "cat",
     "stat" => "stat",
     "grep" => "grep",

     "uname" => "uname",
     "uptime" => "uptime",

     "df" => "df",

     "sensors" => "sensors",
     "nvidia-sensors" => "nvidia-smi",

     "sudo" => "sudo",
     "smartctl" => "smartctl",
    );
   }

   $this->findExecutables($db["host"]);
  }

  function findExecutables($host)
  {
   global $dashboardStorageExecutables; //store globally

   if(!$dashboardStorageExecutables)
    $dashboardStorageExecutables=Array();

   if(!isset($dashboardStorageExecutables[$host]))
    $dashboardStorageExecutables[$host]=Array();

   if($dashboardStorageExecutables[$host])
   {
    $this->execs=$dashboardStorageExecutables[$host];
    return true;
   }

   $cmd="";
   //find executables
   foreach($this->execs as $exec)
    $cmd.=($cmd?" ; ":"")."which ".$exec;

   if(!$this->cli->exec($cmd))
   {
    $this->e->add($this->cli->e);
    return false;
   }

   $l=sizeof($this->cli->output);
   foreach($this->execs as $key => $exec)
   {
    $found=false;
    for($c=0;$c<$l;$c++)
    {
     if(preg_match("!".$exec."$!", $this->cli->output[$c]))
     {
      $this->execs[$key]=$this->cli->output[$c];
      $found=true;
     }
    }

    if(!$found)
    {
     $this->e->add("Dashboard command: \"".$this->execs[$key]."\" not found", "debug");
     $this->execs[$key]=false;
    }
   }

   $dashboardStorageExecutables[$host]=$this->execs;

   return true;
  }


  function table()
  {
   if(!isset($this->tables[$this->data->module]))
    return false;

   return true;
  }


  function get($gather="data")
  {
   if(!isset($this->tables[$this->data->module]))
   {
    $this->e->add("Dashboard storage: module \"".$this->data->module."\" not supported", "system");
    return false;
   }

   $func=$this->tables[$this->data->module]["get"];

   if(($set=$this->$func())===false)
    return false;

   if(!$this->_filterDataset($set, $gather))
    return false;

   if(!$this->_gatherFilterRights())
    return false;

   if($gather=="count"||$gather=="both")
   {
    if(!$this->_setCount())
     return false;

    if($gather=="count")
     return true;
   }

   if(!$this->_sort())
    return false;

   if(!$this->_filterShowTotal())
    return false;

   return true;
  }


  function cli($cmd /*, $cmd, $cmd, etc*/)
  {
   //get arguments
   $cmds=func_get_args();
   if(sizeof($cmds)==1&&is_array($cmds[0])) //first arg contains all cmds
    $cmds=$cmds[0];

   //build up command
   if($this->method=="snmp")
   {
    $options=" -v 3 -u ".$this->username;
    
    if($this->password)
    {
     $secLevel="Auth";
     $options.=" -A ".$this->password;
    }
    else
     $secLevel="NoAuth";

    if($this->passphrase)
    {
     $secLevel.="Priv";
     $options.=" -X ".$this->passphrase;
    }
    else
     $secLevel.="NoPriv";

    $options.=" -l ".$secLevel." ".$this->hostname.":".$this->port;

    $cli=$this->execs["snmpget"].$options;
    foreach($cmds as $cmd)
    {
     $cli.=" NET-SNMP-EXTEND-MIB::nsExtendCacheTime.\\\"".$cmd."\\\"";
     $cli.=" NET-SNMP-EXTEND-MIB::nsExtendOutputFull.\\\"".$cmd."\\\"";
    }

    //run command
    $this->cli->e->clear();
    if(!$this->cli->exec($cli, false))
    {
     $this->e->add($this->cli->e);
     return false;
    }

    //parse output
    $output=implode("\n", $this->cli->output);

    //check for errors
    if(strstr($output, "snmpget: Unknown host"))
    {
     $this->e->add("SNMP connection to ".$this->hostname." failed, unknown hostname", "error");
     return false;
    }
    if(strstr($output, "snmpget: Unknown user name"))
    {
     $this->e->add("SNMP connection to ".$this->hostname." failed, unknown username: ".$this->username, "error");
     return false;
    }
    if(strstr($output, "Error: passphrase chosen is below the length requirements of the USM"))
    {
     $this->e->add("SNMP connection to ".$this->hostname." failed, password / passphrase too short", "error");
     return false;
    }
    if(strstr($output, "snmpget: Authentication failure"))
    {
     $this->e->add("SNMP connection to ".$this->hostname." failed, username / password incorrect", "error");
     return false;
    }
    if(strstr($output, "Timeout: No Response from")||strstr($output, "snmpget: Timeout"))
    {
     $this->e->add("SNMP connection to ".$this->hostname." failed, timeout", "error");
     return false;
    }

    //check cache time
    if(preg_match_all("/NET-SNMP-EXTEND-MIB::nsExtendCacheTime.\"([^\"]+)\".*(\d+)/", $output, $m))
    {
     //set cache time to 1 second for all entries that aren't on 1 yet
     $cli="";
     $l=sizeof($m[0]);
     for($c=0;$c<$l;$c++)
     {
      if($m[2][$c]>1)
      {
       $cli.=" NET-SNMP-EXTEND-MIB::nsExtendCacheTime.\\\"".$m[1][$c]."\\\" i 1";
       $setcache=true;
      }
     }

     if($cli) //set cache changes
     {
      $cli=$this->execs["snmpset"].$options.$cli;
      //run command
      if(!$this->cli->exec($cli, false))
      {
       $this->e->add($this->cli->e);
       return false;
      }
     }

     //remove cache time from output
     $output=preg_replace("/NET-SNMP-EXTEND-MIB::nsExtendCacheTime.+\n/", "", $output);
    }

    //replace snmp id with CMD headers
    $output=preg_replace("/NET-SNMP-EXTEND-MIB::nsExtendOutputFull.\"([^\"]+)\" = STRING: /", "===== CMD = \\1 =====\n", $output);
    $output=explode("\n", $output);
   }
   else //normal/ssh method
   {
    $cli="";
    foreach($cmds as $cmd)
    {
     $cli.=($cli?" ; ":"").$this->execs["echo"]." ===== CMD = ".$cmd." =====";

     //get disk name
     if(strpos($cmd, "disk-stats-sd")!==false)
     {
      $disk=substr($cmd, 11);
      $cmd="disk-stats";
     }

     $exec=false;
     switch($cmd)
     {
      //system
      case "hostname":
      {
       $cli.=" ; ".$this->execs["uname"]." --nodename";
       break;
      }
      case "processor":
      {
       $cli.=" ; ".$this->execs["uname"]." --processor";
       break;
      }
      case "hardware":
      {
       $cli.=" ; ".$this->execs["uname"]." --hardware-platform";
       break;
      }
      case "machine":
      {
       $cli.=" ; ".$this->execs["uname"]." --machine";
       break;
      }
      case "os":
      {
       $cli.=" ; ".$this->execs["uname"]." --operating-system";
       break;
      }

      //kernel
      case "kernel-name":
      {
       $cli.=" ; ".$this->execs["uname"]." --kernel-name";
       break;
      }
      case "kernel-release":
      {
       $cli.=" ; ".$this->execs["uname"]." --kernel-release";
       break;
      }
      case "kernel-version":
      {
       $cli.=" ; ".$this->execs["uname"]." --kernel-version";
       break;
      }

      //distro
      case "slackware-version":
      {
       $cli.=" ; ".$this->execs["cat"]." /etc/slackware-version";
       break;
      }
      case "slackware-lastupdate":
      {
       $cli.=" ; ".$this->execs["cat"]." /var/lib/slackpkg/LASTUPDATE";
       break;
      }
      case "slackware-lastupdated":
      {
       $cli.=" ; ".$this->execs["stat"]." --format %Z /var/lib/slackpkg/LASTUPDATE";
       break;
      }

      //uptime/load
      case "uptime":
      {
       $cli.=" ; ".$this->execs["uptime"];
       break;
      }
      case "load":
      {
       $cli.=" ; ".$this->execs["cat"]." /proc/loadavg";
       break;
      }

      //memory
      case "memory":
      {
       $cli.=" ; ".$this->execs["cat"]." /proc/meminfo";
       break;
      }

      //network
      case "network":
      {
       $cli.=" ; ".$this->execs["cat"]." /proc/net/dev";
       break;
      }

      //disk
      case "disk-space":
      {
       $cli.=" ; ".$this->execs["df"]." -T -B1 -P -l";
       break;
      }

      case "disk-stats-list":
      {
       $cli.=" ; ".$this->execs["sudo"]." ".$this->execs["smartctl"]." --scan";
       break;
      }
      case "disk-stats":
      {
       $cli.=" ; ".$this->execs["sudo"]." ".$this->execs["smartctl"]." --attributes --info /dev/".$disk;
       break;
      }

      //sensors
      case "sensors":
      {
       $cli.=" ; ".$this->execs["sensors"];
       break;
      }
      case "nvidia-sensors":
      {
       $cli.=" ; ".$this->execs["nvidia-sensors"];
       break;
      }
      case "nvidia-sensors-list":
      {
       $cli.=" ; ".$this->execs["nvidia-sensors"]." -L";
       break;
      }
      default:
      {
       $this->e->add("Dashboard get info failed: \"".$cmd."\" command unknown", "system");
       return false;
      }
     }
    }

    //run command
    $this->cli->e->clear();
    if(!$this->cli->exec($cli))
    {
     $this->e->add($this->cli->e);
     return false;
    }
   
    $output=$this->cli->output;
   }

   return $output;
  }

//SYSTEM INFO

  function getSystem()
  {
   $row=0;
   $rows=Array();
   $this->clear();

   if(!($output=$this->cli(
    "hostname",
    "processor",
    "hardware",
    "machine",
    "os",
    "kernel-name",
    "kernel-release",
    "kernel-version",
    "slackware-version",
    "slackware-lastupdate",
    "slackware-lastupdated"
   )))
    return false;

   $rows[$row]=Array();

   $rows[$row]["hostname"]=      (isset($output[1])?$output[1]:"");
   $rows[$row]["processor"]=     (isset($output[3])?$output[3]:"");
   $rows[$row]["hardware"]=      (isset($output[5])?$output[5]:"")." ".(isset($output[7])?$output[7]:"");
   $rows[$row]["os"]=            (isset($output[9])?$output[9]:"");

   $rows[$row]["kernel"]=        (isset($output[11])?$output[11]:"");
   $rows[$row]["kernelrelease"]= (isset($output[13])?$output[13]:"");
   $rows[$row]["kernelversion"]= (isset($output[15])?$output[15]:"");

   if(isset($output[17]))
   {
    if(preg_match("/(.*)\s+([\d\.]+)/", $output[17], $m))
    {
     $rows[$row]["distro"]=$m[1];
     $rows[$row]["distroversion"]=$m[2];
    }
    else
    {
     $rows[$row]["distro"]=$output[17];
     $rows[$row]["distroversion"]="";
    }
   }

   if(isset($output[19]))
    $rows[$row]["distrolastupdate"]=date($this->data->format["sql"]["datetimeformat"], $output[19]);
   else
    $rows[$row]["distrolastupdate"]="";

   if(isset($output[21]))
    $rows[$row]["distrolastupdated"]=date($this->data->format["sql"]["datetimeformat"], $output[21]);
   else
    $rows[$row]["distrolastupdated"]="";

   $rows[$row]["updated"]=date($this->data->format["sql"]["datetimeformat"]);

   return $rows;
  }


//UPTIME / LOAD

  function getUptime()
  {
   $row=0;
   $rows=Array();
   $this->clear();

   if(!($output=$this->cli("uptime", "load")))
    return false;
 
   if(!preg_match("/up (.*) (\d+) user/", $output[1], $uptime))
   {
    $this->e->add("Dashboard get uptime failed: unexpected output", "system");
    return false;
   }

   if(!preg_match("/([0-9\.]+) ([0-9\.]+) ([0-9\.]+) ([0-9]+)\/([0-9]+)/", $output[3], $load))
   {
    $this->e->add("Dashboard get uptime failed: unexpected output", "system");
    return false;
   }

   $rows[$row]=Array();

   //uptime
   if(preg_match("/(\d+) day/", $uptime[1], $m))
    $rows[$row]["updays"]=$m[1];
   else
    $rows[$row]["updays"]=0;

   if(preg_match("/(\d+):(\d+)/", $uptime[1], $m))
    $rows[$row]["uptime"]=sprintf("%02d", $m[1]).":".$m[2].":00";
   else if(preg_match("/(\d+) min/", $uptime[1], $m))
    $rows[$row]["uptime"]="00:".$m[1].":00";
   else
    $rows[$row]["uptime"]="00:00:00";

   $rows[$row]["users"]=$uptime[2];

   //load
   $rows[$row]["load1"]=$load[1];
   $rows[$row]["load5"]=$load[2];
   $rows[$row]["load15"]=$load[3];
   $rows[$row]["activeprocs"]=$load[4];
   $rows[$row]["totalprocs"]=$load[5];

   $rows[$row]["updated"]=date($this->data->format["sql"]["datetimeformat"]);


   return $rows;
  }


//MEMORY

  function getMemory()
  {
   $row=0;
   $rows=Array();
   $this->clear();

   if(!($output=$this->cli("memory")))
    return false;

   $rows[$row]=Array();
   foreach($output as $line)
   {
    if(preg_match("!^([^:]+):\s+(\d+)!", $line, $m))
    {
     $key=strtolower($m[1]);
     $value=$m[2]."000"; //kilo bytes

     if($key=="memtotal")
      $rows[$row]["memorytotal"]=$value;
     else if($key=="memfree")
      $rows[$row]["memoryavailable"]=$value;
     else if($key=="buffers")
      $rows[$row]["memorybuffers"]=$value;

     else if($key=="swaptotal")
      $rows[$row]["swaptotal"]=$value;
     else if($key=="swapfree")
      $rows[$row]["swapavailable"]=$value;

     else if($key=="swapcached")
      $rows[$row]["swapcached"]=$value;
     else if($key=="cached")
      $rows[$row]["memorycached"]=$value;
    }
   }

   //used
   if(isset($rows[$row]["memorytotal"])&&isset($rows[$row]["memoryavailable"]))
    $rows[$row]["memoryused"]=$rows[$row]["memorytotal"]-$rows[$row]["memoryavailable"];

   if(isset($rows[$row]["swaptotal"])&&isset($rows[$row]["swapavailable"]))
    $rows[$row]["swapused"]=$rows[$row]["swaptotal"]-$rows[$row]["swapavailable"];

   //percentage
   $rows[$row]["memoryavailable_percentage"]=round((($rows[$row]["memoryavailable"])/$rows[$row]["memorytotal"])*100, 1);
   $rows[$row]["memoryused_percentage"]=round((($rows[$row]["memoryused"])/$rows[$row]["memorytotal"])*100, 1);
   $rows[$row]["memorycached_percentage"]=round((($rows[$row]["memorycached"])/$rows[$row]["memorytotal"])*100, 1);
   $rows[$row]["memorybuffers_percentage"]=round((($rows[$row]["memorybuffers"])/$rows[$row]["memorytotal"])*100, 1);

   if(!preg_match("/^0+$/", $rows[$row]["swaptotal"]))
   {
    $rows[$row]["swapavailable_percentage"]=round((($rows[$row]["swapavailable"])/$rows[$row]["swaptotal"])*100, 1);
    $rows[$row]["swapused_percentage"]=round((($rows[$row]["swapused"])/$rows[$row]["swaptotal"])*100, 1);
    $rows[$row]["swapcached_percentage"]=round((($rows[$row]["swapcached"])/$rows[$row]["swaptotal"])*100, 1);
   }
   else
   {
    $rows[$row]["swapavailable_percentage"]=0;
    $rows[$row]["swapused_percentage"]=0;
    $rows[$row]["swapcached_percentage"]=0;
   }

   //updated
   $rows[$row]["updated"]=date($this->data->format["sql"]["datetimeformat"]);

   return $rows;
  }


//NETWORK

  function getNetwork()
  {
   $row=0;
   $rows=Array();
   $this->clear();

   if(!($output=$this->cli("network")))
    return false;

   foreach($output as $line)
   {
    if(preg_match("/^\s*([^\:]+)[^\d]+(\d+)[^\d]+\d+[^\d]+\d+[^\d]+\d+[^\d]+\d+[^\d]+\d+[^\d]+\d+[^\d]+\d+[^\d]+(\d+).*$/", $line, $m))
    {
     $rows[$row]=Array();
     $rows[$row]["device"]=$m[1];

     $rows[$row]["rx"]=$m[2];
     $rows[$row]["tx"]=$m[3];

     $rows[$row]["updated"]=date($this->data->format["sql"]["datetimeformat"]);
     $row++;
    }
   }

   return $rows;
  }


//DISKSPACE

  function getDiskspace()
  {
   $row=0;
   $rows=Array();
   $this->clear();

   if(!($output=$this->cli("disk-space")))
    return false;

   foreach($output as $line)
   {
    if(preg_match("!^([^\s]+)\s+([^\s]+)\s+(\d+)\s+(\d+)\s+(\d+)\s+[^\s]+\s([^\s]+)$!", $line, $m))
    {
     $rows[$row]=Array();
     $rows[$row]["device"]=$m[1];
     $rows[$row]["filesystem"]=$m[2];
     $rows[$row]["mountpoint"]=$m[6];

     $rows[$row]["total"]=$m[3];
     $rows[$row]["used"]=$m[4];
     $rows[$row]["available"]=$m[5];
     $rows[$row]["missing"]=($rows[$row]["total"]-$rows[$row]["used"])-$rows[$row]["available"];

     if($rows[$row]["missing"]==0)
      $rows[$row]["missing"]=0;

     $rows[$row]["used_percentage"]=round((($rows[$row]["used"])/$rows[$row]["total"])*100, 1);
     $rows[$row]["available_percentage"]=round((($rows[$row]["available"])/$rows[$row]["total"])*100, 1);
     $rows[$row]["missing_percentage"]=round((($rows[$row]["missing"])/$rows[$row]["total"])*100, 1);

     $rows[$row]["updated"]=date($this->data->format["sql"]["datetimeformat"]);
     $row++;
    }
   }

   return $rows;
  }


//DISKSTATS

  function getDiskstats()
  {
   $row=0;
   $rows=Array();
   $this->clear();

   if(!($output=$this->cli("disk-stats-list")))
    return false;

   $devices=Array();
   if(stristr($output[0], "no askpass program specified"))
   {
    $this->e->add("Unable to scan for disks, please make sure smartctl --scan is in sudoers file", "hack", "/etc/sudoers: ".$this->cli->whoami()." ALL=NOPASSWD:".$this->execs["smartctl"]." --scan");
    return false;
   }

   foreach($output as $line)
   {
    if(preg_match("!^(/[a-z0-9]+/[a-z0-9]+)!", $line, $m))
     $devices[]=$m[1];
   }

   $cmds=Array();
   foreach($devices as $device)
    $cmds[]="disk-stats-".substr($device, strrpos($device, "/")+1);

   unset($device);

   if($output=$this->cli($cmds))
   {
    foreach($output as $line)
    {
     if(preg_match("/^===== CMD = disk-stats-(sd[a-z]) =====$/", $line, $m)) //device
     {
      $device="/dev/".$m[1];
      $family=false;
      $model=false;
     }

     if(preg_match("/^Model Family:\s+ (.+)/i", $line, $m)) //family
      $family=$m[1];
     else if(preg_match("/^Device Model:\s+ (.+)/i", $line, $m)) //model
      $model=$m[1];
     else if(preg_match("/^\s*([0-9]+) ([a-z_]+).*Always.*\s(\+|\-)?([0-9]+)( \(.+\))?$/i", $line, $m)) //attribute
     {
      //set sensor
      $rows[$row]=Array();
      $rows[$row]["device"]=$device;
      $rows[$row]["family"]=$family;
      $rows[$row]["model"]=$model;

      $rows[$row]["attributenr"]=$m[1];
      $rows[$row]["attribute"]=str_replace("_", " ", $m[2]);
      $rows[$row]["deviceattribute"]=$rows[$row]["device"].":".$rows[$row]["attributenr"];

      if($m[3]=="-") //negative in case of minus sign
       $rows[$row]["value"]=$m[4]*-1;
      else
       $rows[$row]["value"]=$m[4];

      if($rows[$row]["attributenr"]==190||$rows[$row]["attributenr"]==194) //temperature
       $rows[$row]["unit"]="C";
      else
       $rows[$row]["unit"]="";

      //detect min / max values
      if(isset($m[5])&&preg_match("!Min/Max\s(\+|\-)?([0-9]+)/(\+|\-)?([0-9]+)!", $m[5], $m2))
      {
       if($m2[1]=="-")
        $rows[$row]["value_min"]=$m2[2]*-1;
       else
        $rows[$row]["value_min"]=$m2[2];

       if($m2[3]=="-")
        $rows[$row]["value_max"]=$m2[4]*-1;
       else
        $rows[$row]["value_max"]=$m2[4];
      }

      //updated
      $rows[$row]["updated"]=date($this->data->format["sql"]["datetimeformat"]);
      $row++;
     }
    }
   }

   return $rows;
  }


//SENSORS

  function getSensors()
  {
   $row=0;
   $rows=Array();
   $this->clear();

   if(!($output=$this->cli("sensors", "nvidia-sensors-list", "nvidia-sensors")))
    return false;

   $device="";
   $adapter="";

   //normal sensors
   foreach($output as $line)
   {
    if(strpos($line, "===== CMD = nvidia")===0) //end of sensors section
     break;
    else if(!$line) //next device
    {
     $device="";
     $adapter="";
    }
    else if(preg_match("/^([^\s]+)$/", $line, $m)) //device
     $device=$m[1];
    else if(preg_match("/^Adapter: (.+)$/", $line, $m)) //adapter
     $adapter=$m[1];
    else if($device&&preg_match("/^([^:]+):\s*(\+|\-)?(\d+(\.\d+)?)\s*([^\s]*)(.*)/", $line, $m)) //sensor
    {
     //set sensor
     $rows[$row]=Array();
     $rows[$row]["device"]=$device;
     $rows[$row]["adapter"]=$adapter;

     $rows[$row]["sensor"]=$m[1];
     $rows[$row]["devicesensor"]=$rows[$row]["device"]."->".$rows[$row]["sensor"];

     if($m[2]=="-") //negative in case of minus sign
      $rows[$row]["value"]=$m[3]*-1;
     else
      $rows[$row]["value"]=$m[3];

     $rows[$row]["unit"]=$m[5];

     //detect min / max values
     if(preg_match_all("/(\w+)\s*=\s*(\+|\-)?(\d+(\.\d+)?)/", $m[6], $m2))
     {
      for($c=0;$c<sizeof($m2[0]);$c++)
      {
       //use min/low as minimum value
       if(!isset($rows[$row]["value_min"])&&($m2[1][$c]=="min"||$m2[1][$c]=="low"))
       {
        if($m2[2][$c]=="-")
         $rows[$row]["value_min"]=$m2[3][$c]*-1;
        else
         $rows[$row]["value_min"]=$m2[3][$c];
       }

       //use max / high as maximum value
       if(!isset($rows[$row]["value_max"])&&($m2[1][$c]=="max"||$m2[1][$c]=="high"))
       {
        if($m2[2][$c]=="-")
         $rows[$row]["value_max"]=$m2[3][$c]*-1;
        else
         $rows[$row]["value_max"]=$m2[3][$c];
       }
      }
     }

     //updated
     $rows[$row]["updated"]=date($this->data->format["sql"]["datetimeformat"]);
     $row++;
    }
   }

   //nvidia sensor
   $adapters=Array();

   foreach($output as $line)
    if(preg_match("/^GPU\s+(\d+):\s(.+)\s\(/i", $line, $m)) //next adapter
     $adapters[$m[1]]=$m[2];

   if($adapters)
   {
    foreach($output as $line)
    {
     if(preg_match("/^(\|---|\|===|\+---)/", $line, $m)) //table separator
     {
      $device="";
      $adapter="";
     }
     else if(preg_match("/^\|\s+(\d+)(\.|\s+)/", $line, $m)&&isset($adapters[$m[1]])) //next adapter
     {
      $device=$adapters[$m[1]];
      $adapter="NVIDIA GPU ".$m[1];
     }
     else if($adapter&&preg_match_all("/(N\/A|P\d|(\d+)\s*(%|C|MB|MiB))/", $line, $m))
     {
      $sensors=Array(
       "Fan Speed",
       "Temperature",
       "Power",
       "Usage",
       "Capacity",

       "Memory",
       "Memory used",
       "Memory total",
      );

      if(sizeof($m[0])==8) //memory percentage not available in never versions of nvidia-smi
       array_splice($sensors, 5, 1); //remove Memory

      //set sensors
      foreach($sensors as $c => $sensor)
      {
       if($m[0][$c]!="N/A") //Fan
       {
        $rows[$row]=Array();
        $rows[$row]["device"]=$device;
        $rows[$row]["adapter"]=$adapter;

        $rows[$row]["sensor"]=$sensor;
        $rows[$row]["devicesensor"]=$rows[$row]["device"]."->".$rows[$row]["sensor"];

        $rows[$row]["value"]=$m[2][$c];
        $rows[$row]["unit"]=$m[3][$c];

        //updated
        $rows[$row]["updated"]=date($this->data->format["sql"]["datetimeformat"]);
        $row++;
       }
      }
     }
    }
   }

   return $rows;
  }
 }
?>
